@@ -33,7 +33,7 @@
 #define	FR_RX_SAMPLE_RATE_DIV	 1
 
 // 2 is available.
-// 3 is used below for FR_ATR_CTL.
+// 3 is available.
 
 #define	FR_MASTER_CTRL		 4	// master enable and reset controls
 #  define  bmFR_MC_ENABLE_TX		(1 << 0)
@@ -142,17 +142,4 @@
 #define	FR_ATR_TXVAL_3		30
 #define FR_ATR_RXVAL_3		31
 
-// Auto Transmit/Receive Control Register
-//
-//    3                   2                   1                       
-//  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
-// +-------------------------------------------------------------+-+
-// |                      Reserved (Must be zero)                |E|
-// +-------------------------------------------------------------+-+
-//
-
-#define FR_ATR_CTL		 3
-#  define bmFR_ATR_CTL_EN		(1 <<  0)
-
-
 #endif /* INCLUDED_FPGA_REGS_COMMON_H */
