@@ -38,7 +38,7 @@
 
 char *prog_name;
 
-static bool test_output (usrp_standard_tx *utx, bool forever_p, double ampl,
+static bool test_output (usrp_standard_tx *utx, int max_bytes, double ampl,
 			 bool dc_p, bool counting_p);
 
 static void
@@ -57,6 +57,7 @@ usage ()
   fprintf (stderr, 
 	   "usage: %s [-f] [-v] [-d] [-c] [-a <ampl>][-I <interp>] [-F freq] [-D]\n", prog_name);
   fprintf (stderr, "  [-f] loop forever\n");
+  fprintf (stderr, "  [-M] how many Megabytes to transfer (default 128)\n");
   fprintf (stderr, "  [-v] verbose\n");
   fprintf (stderr, "  [-d] dump registers\n");
   // fprintf (stderr, "  [-l] digital loopback in FPGA\n");
@@ -103,11 +104,11 @@ int
 main (int argc, char **argv)
 {
   bool 	verbose_p = false;
-  bool	forever_p = false;
   bool  dump_regs_p = false;
   bool	dc_p = false;
   // bool  loopback_p = false;
   bool  counting_p = false;
+  int   max_bytes = 128 * (1L << 20);
   int	ch;
   int	which_board = 0;
   int	interp = 16;			// 32.0 MB/sec 
@@ -116,10 +117,10 @@ main (int argc, char **argv)
 
   set_progname (argv[0]);
 
-  while ((ch = getopt (argc, argv, "vfdcI:F:a:D")) != EOF){
+  while ((ch = getopt (argc, argv, "vfdcI:F:a:DM:")) != EOF){
     switch (ch){
     case 'f':
-      forever_p = true;
+      max_bytes = 0;
       break;
 
     case 'v':
@@ -156,6 +157,11 @@ main (int argc, char **argv)
       ampl = strtod (optarg, 0);
       break;
 
+    case 'M':
+      max_bytes = strtol (optarg, 0, 0) * (1L << 20);
+      if (max_bytes < 0) max_bytes = 0;
+      break;
+
     default:
       usage ();
     }
@@ -178,7 +184,9 @@ main (int argc, char **argv)
   fflush (stdout);
   fflush (stderr);
 
-  test_output (utx, forever_p, ampl, dc_p, counting_p);
+  utx->start();		// start data xfers
+
+  test_output (utx, max_bytes, ampl, dc_p, counting_p);
 
   delete utx;
 
@@ -187,14 +195,13 @@ main (int argc, char **argv)
 
 
 static bool
-test_output (usrp_standard_tx *utx, bool forever_p, double ampl,
+test_output (usrp_standard_tx *utx, int max_bytes, double ampl,
 	     bool dc_p, bool counting_p)
 {
-  static const int BUFSIZE = 16 * 1024;
+  static const int BUFSIZE = utx->block_size();
   static const int N = BUFSIZE/sizeof (short);
   short 	   buf[N];
   int		   nbytes = 0;
-  int		   max_bytes = 512 * (1L << 20);
   int		   counter = 0;
 
   static const int    PERIOD = 65;		// any value is valid
@@ -220,7 +227,7 @@ test_output (usrp_standard_tx *utx, bool forever_p, double ampl,
   int 	nunderruns = 0;
   int 	pi = 0;
 
-  for (nbytes = 0; forever_p || nbytes < max_bytes; nbytes += BUFSIZE){
+  for (nbytes = 0; max_bytes == 0 || nbytes < max_bytes; nbytes += BUFSIZE){
 
     if (counting_p){
       for (int i = 0; i < N; i++)
@@ -236,13 +243,14 @@ test_output (usrp_standard_tx *utx, bool forever_p, double ampl,
     }
 
     int	ret = utx->write (buf, sizeof (buf), &underrun);
-    if (ret != sizeof (buf)){
+    if ((unsigned) ret != sizeof (buf)){
       fprintf (stderr, "test_output: error, ret = %d\n", ret);
     }
 
     if (underrun){
       nunderruns++;
       printf ("tx_underrun\n");
+      //printf ("tx_underrun %9d %6d\n", nbytes, nbytes/BUFSIZE);
     }
   }
 
