@@ -104,18 +104,29 @@ static unsigned char common_regs[] = {
 
 usrp_basic::usrp_basic (int which_board, 
 			struct usb_dev_handle *
-			open_interface (struct usb_device *dev))
+			open_interface (struct usb_device *dev),
+			const std::string fpga_filename,
+			const std::string firmware_filename)
   : d_udh (0),
-    d_usb_data_rate (16000000),	// SWAG
+    d_usb_data_rate (16000000),	// SWAG, see below
     d_bytes_per_poll ((int) (POLLING_INTERVAL * d_usb_data_rate)),
     d_verbose (false)
 {
+  /*
+   * SWAG: Scientific Wild Ass Guess.
+   *
+   * d_usb_data_rate is used only to determine how often to poll for over- and under-runs.
+   * We defualt it to 1/2  of our best case.  Classes derived from usrp_basic (e.g., 
+   * usrp_standard_tx and usrp_standard_rx) call set_usb_data_rate() to tell us the
+   * actual rate.  This doesn't change our throughput, that's determined by the signal
+   * processing code in the FPGA (which we know nothing about), and the system limits
+   * determined by libusb, fusb_*, and the underlying drivers.
+   */
   memset (d_fpga_shadows, 0, sizeof (d_fpga_shadows));
 
   usrp_one_time_init ();
 
-  // FIXME allow subclasses to load own code
-  if (!usrp_load_standard_bits (which_board, false))
+  if (!usrp_load_standard_bits (which_board, false, fpga_filename, firmware_filename))
     throw std::runtime_error ("usrp_basic/usrp_load_standard_bits");
 
   struct usb_device *dev = usrp_find_device (which_board);
@@ -124,8 +135,8 @@ usrp_basic::usrp_basic (int which_board,
     throw std::runtime_error ("usrp_basic/usrp_find_device");
   }
 
-  if (!(usrp_usrp1_p (dev) || usrp_usrp2_p (dev))){
-    fprintf (stderr, "usrp_basic: sorry, this code only works with Rev 1 and 2 USRPs\n");
+  if (!(usrp_usrp_p(dev) && usrp_hw_rev(dev) >= 1)){
+    fprintf (stderr, "usrp_basic: sorry, this code only works with USRP revs >= 1\n");
     throw std::runtime_error ("usrp_basic/bad_rev");
   }
 
@@ -230,6 +241,12 @@ usrp_basic::read_i2c (int i2c_addr, int len)
   return std::string (buf, len);
 }
 
+std::string
+usrp_basic::serial_number()
+{
+  return usrp_serial_number(d_udh);
+}
+
 // ----------------------------------------------------------------
 
 bool
@@ -401,8 +418,11 @@ static unsigned char rx_init_regs[] = {
 };
 
 
-usrp_basic_rx::usrp_basic_rx (int which_board, int fusb_block_size, int fusb_nblocks)
-  : usrp_basic (which_board, open_rx_interface),
+usrp_basic_rx::usrp_basic_rx (int which_board, int fusb_block_size, int fusb_nblocks,
+			      const std::string fpga_filename,
+			      const std::string firmware_filename
+			      )
+  : usrp_basic (which_board, open_rx_interface, fpga_filename, firmware_filename),
     d_devhandle (0), d_ephandle (0),
     d_bytes_seen (0), d_first_read (true),
     d_rx_enable (false)
@@ -519,12 +539,15 @@ usrp_basic_rx::stop ()
 }
 
 usrp_basic_rx *
-usrp_basic_rx::make (int which_board, int fusb_block_size, int fusb_nblocks)
+usrp_basic_rx::make (int which_board, int fusb_block_size, int fusb_nblocks,
+		     const std::string fpga_filename,
+		     const std::string firmware_filename)
 {
   usrp_basic_rx *u = 0;
   
   try {
-    u = new usrp_basic_rx (which_board, fusb_block_size, fusb_nblocks);
+    u = new usrp_basic_rx (which_board, fusb_block_size, fusb_nblocks,
+			   fpga_filename, firmware_filename);
     return u;
   }
   catch (...){
@@ -831,8 +854,10 @@ static unsigned char tx_init_regs[] = {
   REG_TX_NCO_FTW_23_16,	0
 };
 
-usrp_basic_tx::usrp_basic_tx (int which_board, int fusb_block_size, int fusb_nblocks)
-  : usrp_basic (which_board, open_tx_interface),
+usrp_basic_tx::usrp_basic_tx (int which_board, int fusb_block_size, int fusb_nblocks,
+			      const std::string fpga_filename,
+			      const std::string firmware_filename)
+  : usrp_basic (which_board, open_tx_interface, fpga_filename, firmware_filename),
     d_devhandle (0), d_ephandle (0),
     d_bytes_seen (0), d_first_write (true),
     d_tx_enable (false)
@@ -876,8 +901,7 @@ usrp_basic_tx::usrp_basic_tx (int which_board, int fusb_block_size, int fusb_nbl
   d_ephandle = d_devhandle->make_ephandle (USRP_TX_ENDPOINT, false,
 					   fusb_block_size, fusb_nblocks);
 
-  _write_fpga_reg(FR_ATR_CTL, 0);		// zero Tx side Auto Transmit/Receive regs
-  _write_fpga_reg(FR_ATR_MASK_0, 0);	
+  _write_fpga_reg(FR_ATR_MASK_0, 0); // zero Tx side Auto Transmit/Receive regs
   _write_fpga_reg(FR_ATR_TXVAL_0, 0);
   _write_fpga_reg(FR_ATR_RXVAL_0, 0);
   _write_fpga_reg(FR_ATR_MASK_2, 0);
@@ -902,8 +926,6 @@ usrp_basic_tx::~usrp_basic_tx ()
   if (!usrp_9862_write_many_all (d_udh, tx_fini_regs, sizeof (tx_fini_regs))){
     fprintf (stderr, "usrp_basic_tx: failed to fini AD9862 TX regs\n");
   }
-
-  _write_fpga_reg(FR_ATR_CTL, 0);	// disable Auto T/R
 }
 
 bool
@@ -946,12 +968,15 @@ usrp_basic_tx::stop ()
 }
 
 usrp_basic_tx *
-usrp_basic_tx::make (int which_board, int fusb_block_size, int fusb_nblocks)
+usrp_basic_tx::make (int which_board, int fusb_block_size, int fusb_nblocks,
+		     const std::string fpga_filename,
+		     const std::string firmware_filename)
 {
   usrp_basic_tx *u = 0;
   
   try {
-    u = new usrp_basic_tx (which_board, fusb_block_size, fusb_nblocks);
+    u = new usrp_basic_tx (which_board, fusb_block_size, fusb_nblocks,
+			   fpga_filename, firmware_filename);
     return u;
   }
   catch (...){
