@@ -67,7 +67,9 @@ protected:
   unsigned int		 d_fpga_shadows[MAX_REGS];
 
   usrp_basic (int which_board,
-	      struct usb_dev_handle *open_interface (struct usb_device *dev));
+	      struct usb_dev_handle *open_interface (struct usb_device *dev),
+	      const std::string fpga_filename = "",
+	      const std::string firmware_filename = "");
 
   /*!
    * \brief advise usrp_basic of usb data rate (bytes/sec)
@@ -190,6 +192,13 @@ public:
   bool set_adc_buffer_bypass (int which, bool bypass);
 
 
+  /*!
+   * \brief return the usrp's serial number.
+   *
+   * \returns non-zero length string iff successful.
+   */
+  std::string serial_number();
+
   // ----------------------------------------------------------------
   // Low level implementation routines.
   // You probably shouldn't be using these...
@@ -314,7 +323,10 @@ protected:
    */
   usrp_basic_rx (int which_board,
 		 int fusb_block_size=0,
-		 int fusb_nblocks=0);  // throws if trouble
+		 int fusb_nblocks=0,
+		 const std::string fpga_filename = "",
+		 const std::string firmware_filename = ""
+		 );  // throws if trouble
 
   bool set_rx_enable (bool on);
   bool rx_enable () const { return d_rx_enable; }
@@ -338,7 +350,10 @@ public:
    */
   static usrp_basic_rx *make (int which_board,
 			      int fusb_block_size=0,
-			      int fusb_nblocks=0);
+			      int fusb_nblocks=0,
+			      const std::string fpga_filename = "",
+			      const std::string firmware_filename = ""
+			      );
 
   // MANIPULATORS
 
@@ -547,7 +562,10 @@ private:
    */
   usrp_basic_tx (int which_board,
 		 int fusb_block_size=0,
-		 int fusb_nblocks=0);		// throws if trouble
+		 int fusb_nblocks=0,
+		 const std::string fpga_filename = "",
+		 const std::string firmware_filename = ""
+		 );		// throws if trouble
 
   bool set_tx_enable (bool on);
   bool tx_enable () const { return d_tx_enable; }
@@ -570,8 +588,10 @@ public:
    *                         Use zero for a reasonable default.
    * \param fusb_nblocks     number of fast usb URBs to allocate.  Use zero for a reasonable default. 
    */
-  static usrp_basic_tx *make (int which_board, int fusb_block_size=0, int fusb_nblocks=0);
-
+  static usrp_basic_tx *make (int which_board, int fusb_block_size=0, int fusb_nblocks=0,
+			      const std::string fpga_filename = "",
+			      const std::string firmware_filename = ""
+			      );
 
   // MANIPULATORS
 
