@@ -31,6 +31,11 @@
 #include <ad9862.h>
 
 
+static const int OLD_CAPS_VAL = 0xaa55ff77;
+static const int DEFAULT_CAPS_VAL = ((2 << bmFR_RB_CAPS_NDUC_SHIFT)
+				     | (2 << bmFR_RB_CAPS_NDDC_SHIFT)
+				     | bmFR_RB_CAPS_RX_HAS_HALFBAND);
+
 // #define USE_FPGA_TX_CORDIC
 
 
@@ -38,8 +43,6 @@ using namespace ad9862;
 
 #define NELEM(x) (sizeof (x) / sizeof (x[0]))
 
-// The FPGA NCOs use a signed 32-bit frequency tuning word,
-// but we keep only the most significant 14 bits.
 
 static unsigned int
 compute_freq_control_word_fpga (double master_freq, double target_freq,
@@ -48,8 +51,9 @@ compute_freq_control_word_fpga (double master_freq, double target_freq,
   static const int NBITS = 14;
   
   int	v = (int) rint (target_freq / master_freq * pow (2.0, 32.0));
-  // v += 1 << (32 - NBITS - 1);		// add 1/2
-  v = (v >> (32 - NBITS)) << (32 - NBITS);	// keep only top NBITS
+
+  if (0)
+    v = (v >> (32 - NBITS)) << (32 - NBITS);	// keep only top NBITS
 
   *actual_freq = v * master_freq / pow (2.0, 32.0);
 
@@ -84,6 +88,51 @@ compute_freq_control_word_9862 (double master_freq, double target_freq,
   return (unsigned int) v;
 }
 
+// ----------------------------------------------------------------
+
+usrp_standard_common::usrp_standard_common(usrp_basic *parent)
+{
+  // read new FPGA capability register
+  if (!parent->_read_fpga_reg(FR_RB_CAPS, &d_fpga_caps)){
+    fprintf (stderr, "usrp_standard_common: failed to read FPGA cap register.\n");
+    throw std::runtime_error ("usrp_standard_common::ctor");
+  }
+  // If we don't have the cap register, set the value to what it would
+  // have had if we did have one ;)
+  if (d_fpga_caps == OLD_CAPS_VAL)
+    d_fpga_caps = DEFAULT_CAPS_VAL;
+
+  if (0){
+    fprintf(stdout, "has_rx_halfband = %d\n", has_rx_halfband());
+    fprintf(stdout, "nddcs           = %d\n", nddcs());
+    fprintf(stdout, "has_tx_halfband = %d\n", has_tx_halfband());
+    fprintf(stdout, "nducs           = %d\n", nducs());
+  }
+}
+
+bool
+usrp_standard_common::has_rx_halfband() const
+{
+  return (d_fpga_caps & bmFR_RB_CAPS_RX_HAS_HALFBAND) ? true : false;
+}
+
+int
+usrp_standard_common::nddcs() const
+{
+  return (d_fpga_caps & bmFR_RB_CAPS_NDDC_MASK) >> bmFR_RB_CAPS_NDDC_SHIFT;
+}
+
+bool
+usrp_standard_common::has_tx_halfband() const
+{
+  return (d_fpga_caps & bmFR_RB_CAPS_TX_HAS_HALFBAND) ? true : false;
+}
+
+int
+usrp_standard_common::nducs() const
+{
+  return (d_fpga_caps & bmFR_RB_CAPS_NDUC_MASK) >> bmFR_RB_CAPS_NDUC_SHIFT;
+}
 
 // ----------------------------------------------------------------
 
@@ -99,8 +148,13 @@ real_rx_mux_value (int mux, int nchan)
 usrp_standard_rx::usrp_standard_rx (int which_board,
 				    unsigned int decim_rate,
 				    int nchan, int mux, int mode,
-				    int fusb_block_size, int fusb_nblocks)
-  : usrp_basic_rx (which_board, fusb_block_size, fusb_nblocks),
+				    int fusb_block_size, int fusb_nblocks,
+				    const std::string fpga_filename,
+				    const std::string firmware_filename
+				    )
+  : usrp_basic_rx (which_board, fusb_block_size, fusb_nblocks,
+		   fpga_filename, firmware_filename),
+    usrp_standard_common(this),
     d_nchan (1), d_sw_mux (0x0), d_hw_mux (0x0)
 {
   if (!set_format(make_format())){
@@ -160,14 +214,18 @@ usrp_standard_rx *
 usrp_standard_rx::make (int which_board,
 			unsigned int decim_rate,
 			int nchan, int mux, int mode,
-			int fusb_block_size, int fusb_nblocks)
+			int fusb_block_size, int fusb_nblocks,
+			const std::string fpga_filename,
+			const std::string firmware_filename
+			)
 {
   usrp_standard_rx *u = 0;
   
   try {
     u = new usrp_standard_rx (which_board, decim_rate,
 			      nchan, mux, mode,
-			      fusb_block_size, fusb_nblocks);
+			      fusb_block_size, fusb_nblocks,
+			      fpga_filename, firmware_filename);
     return u;
   }
   catch (...){
@@ -191,7 +249,8 @@ usrp_standard_rx::set_decim_rate(unsigned int rate)
 		     * (2 * sizeof (short)));
 
   bool s = disable_rx ();
-  bool ok = _write_fpga_reg (FR_DECIM_RATE, d_decim_rate/2 - 1);
+  int v = has_rx_halfband() ? d_decim_rate/2 - 1 : d_decim_rate - 1;
+  bool ok = _write_fpga_reg (FR_DECIM_RATE, v);
   restore_rx (s);
   return ok;
 }
@@ -201,6 +260,9 @@ bool usrp_standard_rx::set_nchannels (int nchan)
   if (!(nchan == 1 || nchan == 2 || nchan == 4))
     return false;
 
+  if (nchan > nddcs())
+    return false;
+
   d_nchan = nchan;
 
   return write_hw_mux_reg ();
@@ -460,8 +522,12 @@ real_tx_mux_value (int mux, int nchan)
 usrp_standard_tx::usrp_standard_tx (int which_board,
 				    unsigned int interp_rate,
 				    int nchan, int mux,
-				    int fusb_block_size, int fusb_nblocks)
-  : usrp_basic_tx (which_board, fusb_block_size, fusb_nblocks),
+				    int fusb_block_size, int fusb_nblocks,
+				    const std::string fpga_filename,
+				    const std::string firmware_filename
+				    )
+  : usrp_basic_tx (which_board, fusb_block_size, fusb_nblocks, fpga_filename, firmware_filename),
+    usrp_standard_common(this),
     d_sw_mux (0x8), d_hw_mux (0x81)
 {
   if (!usrp_9862_write_many_all (d_udh, tx_regs_use_nco, sizeof (tx_regs_use_nco))){
@@ -519,13 +585,17 @@ usrp_standard_tx *
 usrp_standard_tx::make (int which_board,
 			unsigned int interp_rate,
 			int nchan, int mux,
-			int fusb_block_size, int fusb_nblocks)
+			int fusb_block_size, int fusb_nblocks,
+			const std::string fpga_filename,
+			const std::string firmware_filename
+			)
 {
   usrp_standard_tx *u = 0;
   
   try {
     u = new usrp_standard_tx (which_board, interp_rate, nchan, mux,
-			      fusb_block_size, fusb_nblocks);
+			      fusb_block_size, fusb_nblocks,
+			      fpga_filename, firmware_filename);
     return u;
   }
   catch (...){
@@ -566,6 +636,9 @@ usrp_standard_tx::set_nchannels (int nchan)
   if (!(nchan == 1 || nchan == 2))
     return false;
 
+  if (nchan > nducs())
+    return false;
+
   d_nchan = nchan;
   return write_hw_mux_reg ();
 }
